@@ -15,7 +15,7 @@ has migrations      => sub {
 };
 has url             => sub { Mojo::MySQL5::URL->new('mysql:///test') };
 
-our $VERSION = '0.02';
+our $VERSION = '0.03';
 
 sub db {
   my $self = shift;
@@ -27,7 +27,8 @@ sub db {
   my $db = Mojo::MySQL5::Database->new(connection => $c, mysql => $self);
 
   if (!$c) {
-    $db->connect($self->url);
+    $db->connect;
+    croak 'connect failed' unless $db->connection->_state eq 'idle';
     $self->emit(connection => $db);
   }
   return $db;
@@ -38,6 +39,9 @@ sub from_string {
   my $url = Mojo::MySQL5::URL->new($str);
   croak qq{Invalid MySQL connection string "$str"} unless $url->protocol eq 'mysql';
 
+  $url->options->{utf8} = 1 unless exists $url->options->{utf8};
+  $url->options->{found_rows} = 1 unless exists $url->options->{found_rows};
+
   return $self->url($url);
 }
 
@@ -209,10 +213,6 @@ C<5>.
 
 =head2 migrations
 
-MySQL does not support nested transactions and DDL transactions.
-DDL statements cause implicit C<COMMIT>.
-B<Therefore, migrations should be used with extreme caution.> 
-
   my $migrations = $mysql->migrations;
   $mysql         = $mysql->migrations(Mojo::MySQL5::Migrations->new);
 
@@ -222,6 +222,10 @@ easily.
   # Load migrations from file and migrate to latest version
   $mysql->migrations->from_file('/home/sri/migrations.sql')->migrate;
 
+MySQL does not support nested transactions and DDL transactions.
+DDL statements cause implicit C<COMMIT>. C<ROLLBACK> will be called if
+migration script fails, but not everything will be reverted.
+
 =head2 url
 
   my $url = $mysql->url;
@@ -321,8 +325,6 @@ This is the class hierarchy of the L<Mojo::MySQL5> distribution.
 
 =item * L<Mojo::MySQL5::URL>
 
-=item * L<Mojo::MySQL5::Util>
-
 =back
 
 =head1 AUTHOR
