@@ -6,7 +6,6 @@ use Encode qw(_utf8_off _utf8_on);
 use Digest::SHA qw(sha1);
 use Scalar::Util 'weaken';
 use Mojo::IOLoop;
-use Mojo::MySQL5::Util qw(flag_list flag_set);
 
 has 'url';
 
@@ -160,11 +159,11 @@ sub _send_auth {
   push @flags, 'CONNECT_WITH_DB' if $database;
   push @flags, 'MULTI_STATEMENTS' if $self->url->options->{multi_statements};
   push @flags, 'FOUND_ROWS' if $self->url->options->{found_rows};
-  my $flags = flag_set(CLIENT_CAPABILITY, @flags);
+  my $flags = _flag_set(CLIENT_CAPABILITY, @flags);
 
   warn '>>> AUTH ', $self->{connection_id}, ' #', $self->{seq}, ' state:', $self->_state, "\n",
     ' user:', $username, ' database:', $database,
-    ' flags:', flag_list(CLIENT_CAPABILITY, $flags),
+    ' flags:', _flag_list(CLIENT_CAPABILITY, $flags),
     '(', sprintf('%08X', $flags), ')', "\n" if DEBUG > 1;
 
   _utf8_off $username; _utf8_off $password; _utf8_off $database;
@@ -241,7 +240,7 @@ sub _recv_ok {
   warn '<<< OK ', $self->{connection_id}, ' #', $self->{seq}, ' state:', $self->_state, "\n",
     ' affected:', $self->{affected_rows},
     ' last_insert_id:', $self->{last_insert_id},
-    ' status:', flag_list(SERVER_STATUS, $self->{status_flags}),
+    ' status:', _flag_list(SERVER_STATUS, $self->{status_flags}),
     '(', sprintf('%04X', $self->{status_flags}), ')',
     ' warnings:', $self->{warnings_count}, "\n" if DEBUG > 1;
 
@@ -276,7 +275,7 @@ sub _recv_eof {
 
   warn '<<< EOF ', $self->{connection_id}, ' #', $self->{seq}, ' state:', $self->_state, "\n",
     ' warnings:', $self->{warnings_count},
-    ' status:', flag_list(SERVER_STATUS, $self->{status_flags}),
+    ' status:', _flag_list(SERVER_STATUS, $self->{status_flags}),
     '(', sprintf('%04X', $self->{status_flags}), ')', "\n" if DEBUG > 1;
 
   if ($self->_state eq 'field') {
@@ -328,7 +327,7 @@ sub _recv_field {
     ' type:', REV_DATATYPE->{chr $field->{column_type}}, '(', $field->{column_type}, ')',
     ' length:', $field->{column_length},
     ' charset:', REV_CHARSET->{$field->{character_set}} // 'UNKNOWN', '(', $field->{character_set}, ')',
-    ' flags:', flag_list(FIELD_FLAG, $field->{flags}), '(', $field->{flags}, ')', , "\n" if DEBUG > 1;
+    ' flags:', _flag_list(FIELD_FLAG, $field->{flags}), '(', $field->{flags}, ')', , "\n" if DEBUG > 1;
 }
 
 sub _recv_row {
@@ -374,9 +373,9 @@ sub _recv_handshake {
     ' protocol:', $self->{protocol_version},
     ' version:', $self->{server_version},
     ' connection:', $self->{connection_id},
-    ' status:', flag_list(SERVER_STATUS, $self->{status_flags}),
+    ' status:', _flag_list(SERVER_STATUS, $self->{status_flags}),
     '(', sprintf('%04X', $self->{status_flags}), ')',
-    ' capabilities:', flag_list(CLIENT_CAPABILITY, $self->{capability_flags}),
+    ' capabilities:', _flag_list(CLIENT_CAPABILITY, $self->{capability_flags}),
     '(', sprintf('%08X', $self->{capability_flags}), ')',
     ' auth:', $auth_plugin_name, "\n" if DEBUG > 1;
 
@@ -521,7 +520,6 @@ sub connect {
   );
 
   $self->_ioloop(0)->start unless $cb;
-  die $self->{error_message} if $self->{error_code};
 }
 
 sub disconnect { shift->_cmd('disconnect') }
@@ -542,6 +540,33 @@ sub DESTROY {
   $self->disconnect if $self->_state eq 'idle' and $self->{socket};
 }
 
+# Private util functions
+sub _flag_list($$;$) {
+  my ($list, $data, $sep) = @_;
+  my $i = 0;
+  return join $sep || '|', grep { $data & 1 << $i++ } @$list;
+}
+
+sub _flag_set($;@) {
+  my ($list, @ops) = @_;
+  my ($i, $flags) = (0, 0);
+  foreach my $flag (@$list) {
+    do { $flags |= 1 << $i if $_ eq $flag } for @ops; 
+    $i++;
+  }
+  return $flags;
+}
+
+sub _flag_is($$$) {
+  my ($list, $data, $flag) = @_;
+  my $i = 0;
+  foreach (@$list) {
+    return $data & 1 << $i if $flag eq $_;
+    $i++;
+  }
+  return undef;
+}
+
 1;
 
 =encoding utf8
@@ -555,8 +580,9 @@ Mojo::MySQL5::Connection - TCP connection to MySQL Server
   use Mojo::MySQL5::Conection;
 
   my $c = Mojo::MySQL5::Conection->new(
-    url => 'mysql://test:password@127.0.0.1:3306/test',
-    options => { found_rows => 1, connect_timeout => 2 });
+    url => Mojo::MySQL5->new(
+      'mysql://test:password@127.0.0.1:3306/test?found_rows=1&connect_timeout=2')
+  );
 
   Mojo::IOLoop->delay(
     sub {
@@ -631,13 +657,6 @@ L<Mojo::MySQL5::Conection> implements the following attributes.
 
 MySQL Connection URL.
 
-=head2 options
-
-  my $options = $c->options;
-  $c->options({ connect_timeout => 5, query_timeout => 30, utf => 1 });
-
-Options for Connection.
-
 Supported Options are:
 
 =over 2
@@ -652,37 +671,27 @@ Without C<found_rows>, if you perform a query like
 then the MySQL engine will return 0, because no rows have changed.
 With C<found_rows>, it will return the number of rows that have an id 1.
 
-Default is 1.
-
 =item multi_statements
 
 Enables or disables the flag C<CLIENT_MULTI_STATEMENTS> while connecting to the server.
 If enabled multiple statements separated by semicolon (;) can be send with single
 call to L<query>.
 
-Default is 0.
-
 =item utf8
 
 If enabled default character set is to C<utf8_general_ci> while connecting to the server.
 If disabled C<binary> is the default character set.
 
-Default is 1.
-
 =item connect_timeout
 
 The connect request to the server will timeout if it has not been successful
 after the given number of seconds.
 
-Default is 10.
-
 =item query_timeout
 
 If enabled, the read or write operation to the server will timeout
 if it has not been successful after the given number of seconds.
 
-Default is 0 (disabled).
-
 =back
 
 =head1 METHODS
